@@ -1,10 +1,10 @@
 /*
-  Copyright (c) 1990-1999 Info-ZIP.  All rights reserved.
+  Copyright (c) 1990-2005 Info-ZIP.  All rights reserved.
 
-  See the accompanying file LICENSE, version 1999-Oct-05 or later
+  See the accompanying file LICENSE, version 2004-May-22 or later
   (the contents of which are also included in zip.h) for terms of use.
   If, for some reason, both of these files are missing, the Info-ZIP license
-  also may be found at:  ftp://ftp.cdrom.com/pub/infozip/license.html
+  also may be found at:  ftp://ftp.info-zip.org/pub/infozip/license.html
 */
 /*---------------------------------------------------------------------------
 
@@ -601,17 +601,12 @@ iztimes *t)             /* return value: access, modific. and creation times */
    a file size of -1 */
 {
   struct stat s;        /* results of stat() */
-  char name[NAME_MAX];
-  int len;
 
   AssertStr(f,f)
 
-  len = strlen(f);
-  sstrcpy(name, f);
+  if (strlen(f) == 0) return 0;
 
-  if (len == 0) return 0;
-
-  if (SSTAT(name, &s) != 0)
+  if (SSTAT(f, &s) != 0)
              /* Accept about any file kind including directories
               * (stored with trailing : with -r option)
               */
