@@ -1,13 +1,11 @@
 /*
+  Copyright (c) 1990-2006 Info-ZIP.  All rights reserved.
 
- Copyright (C) 1990-1997 Mark Adler, Richard B. Wales, Jean-loup Gailly,
- Onno van der Linden, Christian Spieler and Kai Uwe Rommel.
- Permission is granted to any individual or institution to use, copy, or
- redistribute this software so long as all of the original files are included,
- that it is not sold for profit, and that this copyright notice is retained.
-
+  See the accompanying file LICENSE, version 2005-Feb-10 or later
+  (the contents of which are also included in zip.h) for terms of use.
+  If, for some reason, all these files are missing, the Info-ZIP license
+  also may be found at:  ftp://ftp.info-zip.org/pub/infozip/license.html
 */
-
 #include "zip.h"
 
 #ifndef UTIL    /* little or no material in this file is used by UTIL */
@@ -22,10 +20,14 @@
 #  define FATTR (hidden_files ? FA_HIDDEN+FA_SYSTEM+FA_DIREC : FA_DIREC)
 #  define FFIRST(n,d,a)   findfirst(n,(struct ffblk *)d,a)
 #  define FNEXT(d)        findnext((struct ffblk *)d)
-#  if (!defined(__DJGPP__) || (__DJGPP__ < 2))
+#  if (defined(__TURBOC__) || (defined(__DJGPP__) && (__DJGPP__ >=2)))
+#    if (defined(__DJGPP__) && (__DJGPP__ == 2) && (__DJGPP_MINOR__ == 0))
+#      include <libc/dosio.h>
+#    endif
+#    define GetFileMode(name) _chmod(name, 0)
+#    define SetFileMode(name, attr) _chmod(name, 1, attr)
+#  else /* DJGPP v1.x */
 #    define GetFileMode(name) bdosptr(0x43, (name), 0)
-#  else
-#    include <libc/dosio.h>
 #  endif
 #endif /* __GO32__ || __TURBOC__ */
 
@@ -65,15 +67,13 @@
 #  define ff_ftime        time
 #  define ff_attrib       attr
 #  define GetFileMode(name) __chmod(name, 0, 0)
+#  define SetFileMode(name, attr) __chmod(name, 1, attr)
 #endif /* __EMX__ */
 
+#ifndef SetFileMode
+#  define SetFileMode(name, attr) _dos_setfileattr(name, attr)
+#endif
 
-/* Keep this =>SYNCHRONIZED<= with the corresponding definition in fileio.c */
-#if defined(__GO32__) || defined(__EMX__)
-#define MATCH shmatch
-#else
-#define MATCH dosmatch
-#endif /* __GO32__ */
 
 #define PAD  0
 #define PATH_END '/'
@@ -90,6 +90,11 @@ int GetFileMode OF((char *name));
 local int  initDirSearch OF((char *name, ff_dir *ff_context_p));
 local char *getVolumeLabel OF((int, ulg *, ulg *, time_t *));
 local int  wild_recurse OF((char *, char *));
+local int procname_dos OF((char *n, int caseflag, unsigned attribs));
+local int is_running_on_windows OF((void));
+
+#define MSDOS_INVALID_ATTR      0xFF
+#define getDirEntryAttr(d)      ((d)->ff_attrib)
 
 /* Module level variables */
 extern char *label;
@@ -105,8 +110,7 @@ local ZCONST char wild_match_all[] = "*.*";
 int GetFileMode(char *name)
 {
   unsigned int attr = 0;
-  _dos_getfileattr(name, &attr);
-  return attr;
+  return (_dos_getfileattr(name, &attr) ? -1 : attr);
 }
 #endif /* !GetFileMode */
 
@@ -117,7 +121,7 @@ local int initDirSearch(name, ff_context_p)
   int r;                        /* FFIRST return value */
   char *p, *q;                  /* temporary copy of name, and aux pointer */
 
-  if ((p = malloc(strlen(name) + 5)) == NULL)
+  if ((p = malloc(strlen(name) + (2 + sizeof(wild_match_all)))) == NULL)
     return ZE_MEM;
 
   strcpy(p, name);
@@ -137,7 +141,7 @@ local char *getVolumeLabel(drive, vtime, vmode, vutim)
   int drive;    /* drive name: 'A' .. 'Z' or '\0' for current drive */
   ulg *vtime;   /* volume label creation time (DOS format) */
   ulg *vmode;   /* volume label file mode */
-  time_t *vutim;/* volume label creationtime (UNIX format) */
+  time_t *vutim;/* volume label creation time (UNIX format) */
 
 /* If a volume label exists for the given drive, return its name and
    set its time and mode. The returned name must be static data. */
@@ -167,7 +171,11 @@ local char *getVolumeLabel(drive, vtime, vmode, vutim)
 }
 
 
-#define ONENAMELEN 12
+#ifdef MSDOS16
+#define ONENAMELEN 12   /* no 16-bit compilers supports LFN */
+#else
+#define ONENAMELEN 255
+#endif
 
 /* whole is a pathname with wildcards, wildtail points somewhere in the  */
 /* middle of it.  All wildcards to be expanded must come AFTER wildtail. */
@@ -185,7 +193,7 @@ char *wildtail;
         struct stat s;                          /* dummy buffer for stat() */
 
         if (!LSSTAT(whole, &s))                 /* file exists ? */
-            return procname(whole);
+            return procname(whole, 0);
         else
             return ZE_MISS;                     /* woops, no wildcards! */
     }
@@ -213,12 +221,12 @@ char *wildtail;
         goto ohforgetit;
     }
     subwild = strchr(wildtail + 1, PATH_END);
+    /* this "+ 1" dodges the  ^^^ hole left by *glue == 0 */
     if (subwild != NULL) {
-        /* this "+ 1" dodges the  ^^^ hole left by *glue == 0 */
         *(subwild++) = 0;               /* wildtail = one component pattern */
-        newlen = strlen(whole) + strlen(subwild) + ONENAMELEN + 2;
+        newlen = strlen(whole) + strlen(subwild) + (ONENAMELEN + 2);
     } else
-        newlen = strlen(whole) + ONENAMELEN + 1;
+        newlen = strlen(whole) + (ONENAMELEN + 1);
     if ((newwhole = malloc(newlen)) == NULL) {
         if (glue)
             *glue = plug;
@@ -236,7 +244,7 @@ char *wildtail;
 
     do {
         if (strcmp(dir.ff_name, ".") && strcmp(dir.ff_name, "..")
-                                  && MATCH(wildtail, dir.ff_name)) {
+                                  && MATCH(wildtail, dir.ff_name, 0)) {
             strcpy(newwhole + newlen, dir.ff_name);
             if (subwild) {
                 name = newwhole + strlen(newwhole);
@@ -244,7 +252,7 @@ char *wildtail;
                 strcpy(name, subwild);
                 e = wild_recurse(newwhole, name);
             } else
-                e = procname(newwhole);
+                e = procname_dos(newwhole, 0, getDirEntryAttr(&dir));
             newwhole[newlen] = 0;
             if (e == ZE_OK)
                 amatch = 1;
@@ -277,13 +285,13 @@ char *w;                /* path/pattern to match */
       label = getVolumeLabel((w != NULL && w[1] == ':') ? to_up(w[0]) : '\0',
                              &label_time, &label_mode, &label_utim);
       if (label != NULL)
-        (void)newname(label, 0);
+        (void)newname(label, 0, 0);
       if (w == NULL || (w[1] == ':' && w[2] == '\0')) return ZE_OK;
       /* "zip -$ foo a:" can be used to force drive name */
     }
     /* special handling of stdin request */
     if (strcmp(w, "-") == 0)   /* if compressing stdin */
-        return newname(w, 0);
+        return newname(w, 0, 0);
 
     /* Allocate and copy pattern, leaving room to add "." if needed */
     if ((p = malloc(strlen(w) + 2)) == NULL)
@@ -313,8 +321,10 @@ char *w;                /* path/pattern to match */
     return e;
 }
 
-int procname(n)
+local int procname_dos(n, caseflag, attribs)
 char *n;                /* name to process */
+int caseflag;           /* true to force case-sensitive match */
+unsigned attribs;       /* file attributes, if available */
 /* Process a name or sh expression to operate on (or exclude).  Return
    an error code in the ZE_ class. */
 {
@@ -331,8 +341,20 @@ char *n;                /* name to process */
     return ZE_OK;
 
   if (strcmp(n, "-") == 0)   /* if compressing stdin */
-    return newname(n, 0);
+    return newname(n, 0, caseflag);
   else if (*n == '\0') return ZE_MISS;
+  else if (attribs != MSDOS_INVALID_ATTR)
+  {
+    /* Avoid calling stat() for performance reasons when it is already known
+       (from a previous directory scan) that the passed name corresponds to
+       a "real existing" file.  The only information needed further down in
+       this function is the distinction between directory entries and other
+       (typically normal file) entries.  This distinction can be derived from
+       the file's attributes that the directory lookup has already provided
+       "for free".
+     */
+    s.st_mode = ((attribs & MSDOS_DIR_ATTR) ? S_IFDIR : S_IFREG);
+  }
   else if (LSSTAT(n, &s)
 #ifdef __TURBOC__
            /* For this compiler, stat() succeeds on wild card names! */
@@ -341,12 +363,18 @@ char *n;                /* name to process */
           )
   {
     /* Not a file or directory--search for shell expression in zip file */
-    p = ex2in(n, 0, (int *)NULL);       /* shouldn't affect matching chars */
+    if (caseflag) {
+      p = malloc(strlen(n) + 1);
+      if (p != NULL)
+        strcpy(p, n);
+    } else
+      p = ex2in(n, 0, (int *)NULL);     /* shouldn't affect matching chars */
     m = 1;
     for (z = zfiles; z != NULL; z = z->nxt) {
-      if (MATCH(p, z->iname))
+      if (MATCH(p, z->iname, caseflag))
       {
-        z->mark = pcount ? filter(z->zname) : 1;
+        z->mark = pcount ? filter(z->zname, caseflag) : 1;
+        if (z->mark) z->dosflag = 1;    /* force DOS attribs for incl. names */
         if (verbose)
             fprintf(mesg, "zip diagnostic: %scluding %s\n",
                z->mark ? "in" : "ex", z->name);
@@ -364,7 +392,7 @@ char *n;                /* name to process */
   if ((s.st_mode & S_IFDIR) == 0)
   {
     /* add or remove name of file */
-    if ((m = newname(n, 0)) != ZE_OK)
+    if ((m = newname(n, 0, caseflag)) != ZE_OK)
       return m;
   } else {
     /* Add trailing / to the directory name */
@@ -377,7 +405,7 @@ char *n;                /* name to process */
       a = p + strlen(p);
       if (a[-1] != '/')
         strcpy(a, "/");
-      if (dirnames && (m = newname(p, 1)) != ZE_OK) {
+      if (dirnames && (m = newname(p, 1, caseflag)) != ZE_OK) {
         free((zvoid *)p);
         return m;
       }
@@ -406,7 +434,8 @@ char *n;                /* name to process */
             return ZE_MEM;
           }
           strcat(strcpy(a, p), e);
-          if ((m = procname(a)) != ZE_OK)   /* recurse on name */
+          if ((m = procname_dos(a, caseflag, getDirEntryAttr(d)))
+              != ZE_OK)         /* recurse on name */
           {
             if (m == ZE_MISS)
               zipwarn("name not matched: ", a);
@@ -423,6 +452,13 @@ char *n;                /* name to process */
   return ZE_OK;
 }
 
+int procname(n, caseflag)
+char *n;                /* name to process */
+int caseflag;           /* true to force case-sensitive match */
+{
+  return procname_dos(n, caseflag, MSDOS_INVALID_ATTR);
+}
+
 char *ex2in(x, isdir, pdosflag)
 char *x;                /* external file name */
 int isdir;              /* input: x is a directory */
@@ -437,7 +473,23 @@ int *pdosflag;          /* output: force MSDOS file attributes? */
   dosflag = 1;
 
   /* Find starting point in name before doing malloc */
+  /* Strip drive specification */
   t = *x && *(x + 1) == ':' ? x + 2 : x;
+  /* Strip "//host/share/" part of a UNC name */
+  if ((!strncmp(x,"//",2) || !strncmp(x,"\\\\",2)) &&
+      (x[2] != '\0' && x[2] != '/' && x[2] != '\\')) {
+    n = x + 2;
+    while (*n != '\0' && *n != '/' && *n != '\\')
+      n++;              /* strip host name */
+    if (*n != '\0') {
+      n++;
+      while (*n != '\0' && *n != '/' && *n != '\\')
+        n++;            /* strip `share' name */
+    }
+    if (*n != '\0')
+      t = n + 1;
+  }
+  /* Strip leading "/" to convert an absolute path into a relative path */
   while (*t == '/' || *t == '\\')
     t++;
   /* Skip leading "./" as well */
@@ -495,7 +547,7 @@ ulg d;                  /* dos-style time to change it to */
 
   if ((h = open(f, 0)) != -1)
   {
-    setftime(h, (struct ftime *)&d);
+    setftime(h, (struct ftime *)(void *)&d);
     close(h);
   }
 #else /* !__TURBOC__ && !__GO32__ */
@@ -527,8 +579,9 @@ iztimes *t;             /* return value: access, modific. and creation times */
    a file size of -1 */
 {
   struct stat s;        /* results of stat() */
-  char name[FNMAX];
-  int len = strlen(f), isstdin = !strcmp(f, "-");
+  char *name;
+  unsigned int len = strlen(f);
+  int isstdin = !strcmp(f, "-");
 
   if (f == label) {
     if (a != NULL)
@@ -539,27 +592,36 @@ iztimes *t;             /* return value: access, modific. and creation times */
       t->atime = t->mtime = t->ctime = label_utim;
     return label_time;
   }
-#if defined(__TURBOC__)
-  /* Call tzset() to set timezone correctly (buggy older TC runtime libs) */
-  tzset();
-#endif /* __TURBOC__ */
+
+  if ((name = malloc(len + 1)) == NULL) {
+    ZIPERR(ZE_MEM, "filetime");
+  }
   strcpy(name, f);
   if (name[len - 1] == '/')
     name[len - 1] = '\0';
   /* not all systems allow stat'ing a file with / appended */
 
   if (isstdin) {
-    if (fstat(fileno(stdin), &s) != 0)
+    if (fstat(fileno(stdin), &s) != 0) {
+      free(name);
       error("fstat(stdin)");
+    }
     time((time_t *)&s.st_mtime);       /* some fstat()s return time zero */
-  } else if (LSSTAT(name, &s) != 0)
+  } else if (LSSTAT(name, &s) != 0) {
              /* Accept about any file kind including directories
               * (stored with trailing / with -r option)
               */
+    free(name);
     return 0;
+  }
 
-  if (a != NULL)
+  if (a != NULL) {
     *a = ((ulg)s.st_mode << 16) | (isstdin ? 0L : (ulg)GetFileMode(name));
+#if (S_IFREG != 0x8000)
+    /* kludge to work around non-standard S_IFREG flag used in DJGPP V2.x */
+    if ((s.st_mode & S_IFMT) == S_IFREG) *a |= 0x80000000L;
+#endif
+  }
   if (n != NULL)
     *n = (s.st_mode & S_IFREG) != 0 ? s.st_size : -1L;
   if (t != NULL) {
@@ -568,6 +630,8 @@ iztimes *t;             /* return value: access, modific. and creation times */
     t->ctime = s.st_ctime;
   }
 
+  free(name);
+
   return unix2dostime((time_t *)&s.st_mtime);
 }
 
@@ -586,6 +650,10 @@ int set_extra_field(z, z_utim)
   /* create extra field and change z->att if desired */
 {
 #ifdef USE_EF_UT_TIME
+#ifdef IZ_CHECK_TZ
+  if (!zp_tz_is_valid) return ZE_OK;    /* skip silently if no valid TZ info */
+#endif
+
   if ((z->extra = (char *)malloc(EB_HEADSIZE+EB_UT_LEN(1))) == NULL)
     return ZE_MEM;
 
@@ -713,13 +781,49 @@ void xit(void)
 
 #endif /* !UTIL */
 
+local int is_running_on_windows(void)
+{
+    char * var = getenv("OS");
+
+    /* if the OS env.var says 'Windows_NT' then */
+    /* we're likely running on a variant of WinNT */
+    if ((NULL != var) && (0 == strcmp("Windows_NT", var)))
+    {
+        return 1;
+    }
+
+    /* if the windir env.var is non-null then */
+    /* we're likely running on a variant of Win9x */
+    /* DOS mode of Win9x doesn't define windir, only winbootdir */
+    /* NT's command.com can't see lowercase env. vars */
+    var = getenv("windir");
+    if ((NULL != var) && (0 != var[0]))
+    {
+        return 1;
+    }
+
+    return 0;
+}
+
+void check_for_windows(const char *app)
+{
+    /* Print a warning for users running under Windows */
+    /* to reduce bug reports due to running DOS version */
+    /* under Windows, when Windows version usually works correctly */
+    if (is_running_on_windows())
+    {
+        printf("\nzip warning:  You are running MSDOS %s on Windows.\n"
+               "Try the Windows version before reporting any problems.\n",
+               app);
+    }
+}
 
 #ifndef WINDLL
 /******************************/
 /*  Function version_local()  */
 /******************************/
 
-static ZCONST char CompiledWith[] = "Compiled with %s%s for %s%s%s%s.\n\n";
+static ZCONST char CompiledWith[] = "Compiled with %s%s for %s%s%s.\n\n";
                         /* At module level to keep Turbo C++ 1.0 happy !! */
 
 void version_local()
@@ -729,125 +833,205 @@ void version_local()
     char buf[80];
 #endif
 
-    printf(CompiledWith,
-
-#ifdef __GNUC__
+/* Define the compiler name and version strings */
+#if defined(__GNUC__)
 #  if defined(__DJGPP__)
-      (sprintf(buf, "djgpp v%d / gcc ", __DJGPP__), buf),
-#  elif defined(__GO32__)
-      "djgpp v1.x / gcc ",
-#  elif defined(__EMX__)            /* ...so is __EMX__ (double sigh) */
-      "emx+gcc ",
+    sprintf(buf, "djgpp v%d.%02d / gcc ", __DJGPP__, __DJGPP_MINOR__);
+#    define COMPILER_NAME1      buf
+#  elif defined(__GO32__)         /* __GO32__ is defined as "1" only (sigh) */
+#    define COMPILER_NAME1      "djgpp v1.x / gcc "
+#  elif defined(__EMX__)          /* ...so is __EMX__ (double sigh) */
+#    define COMPILER_NAME1      "emx+gcc "
 #  else
-      "gcc ",
+#    define COMPILER_NAME1      "gcc "
 #  endif
-      __VERSION__,
+#  define COMPILER_NAME2        __VERSION__
 #elif defined(__WATCOMC__)
 #  if (__WATCOMC__ % 10 > 0)
 /* We do this silly test because __WATCOMC__ gives two digits for the  */
 /* minor version, but Watcom packaging prefers to show only one digit. */
-      (sprintf(buf, "Watcom C/C++ %d.%02d", __WATCOMC__ / 100,
-               __WATCOMC__ % 100), buf), "",
+    sprintf(buf, "Watcom C/C++ %d.%02d", __WATCOMC__ / 100,
+            __WATCOMC__ % 100);
 #  else
-      (sprintf(buf, "Watcom C/C++ %d.%d", __WATCOMC__ / 100,
-               (__WATCOMC__ % 100) / 10), buf), "",
+    sprintf(buf, "Watcom C/C++ %d.%d", __WATCOMC__ / 100,
+            (__WATCOMC__ % 100) / 10);
 #  endif
+#  define COMPILER_NAME1        buf
+#  define COMPILER_NAME2        ""
 #elif defined(__TURBOC__)
 #  ifdef __BORLANDC__
-      "Borland C++",
+#    define COMPILER_NAME1      "Borland C++"
 #    if (__BORLANDC__ < 0x0200)
-        " 1.0",
+#      define COMPILER_NAME2    " 1.0"
 #    elif (__BORLANDC__ == 0x0200)   /* James:  __TURBOC__ = 0x0297 */
-        " 2.0",
+#      define COMPILER_NAME2    " 2.0"
 #    elif (__BORLANDC__ == 0x0400)
-        " 3.0",
+#      define COMPILER_NAME2    " 3.0"
 #    elif (__BORLANDC__ == 0x0410)   /* __BCPLUSPLUS__ = 0x0310 */
-        " 3.1",
+#      define COMPILER_NAME2    " 3.1"
 #    elif (__BORLANDC__ == 0x0452)   /* __BCPLUSPLUS__ = 0x0320 */
-        " 4.0 or 4.02",
+#      define COMPILER_NAME2    " 4.0 or 4.02"
 #    elif (__BORLANDC__ == 0x0460)   /* __BCPLUSPLUS__ = 0x0340 */
-        " 4.5",
+#      define COMPILER_NAME2    " 4.5"
 #    elif (__BORLANDC__ == 0x0500)   /* __TURBOC__ = 0x0500 */
-        " 5.0",
+#      define COMPILER_NAME2    " 5.0"
 #    else
-        " later than 5.0",
+#      define COMPILER_NAME2    " later than 5.0"
 #    endif
 #  else
-      "Turbo C",
+#    define COMPILER_NAME1      "Turbo C"
 #    if (__TURBOC__ > 0x0401)
-        "++ later than 3.0"
+#      define COMPILER_NAME2    "++ later than 3.0"
 #    elif (__TURBOC__ == 0x0401)     /* Kevin:  3.0 -> 0x0401 */
-        "++ 3.0",
+#      define COMPILER_NAME2    "++ 3.0"
+#    elif (__TURBOC__ == 0x0296)     /* [662] checked by SPC */
+#      define COMPILER_NAME2    "++ 1.01"
 #    elif (__TURBOC__ == 0x0295)     /* [661] vfy'd by Kevin */
-        "++ 1.0",
+#      define COMPILER_NAME2    "++ 1.0"
+#    elif (__TURBOC__ == 0x0201)     /* Brian:  2.01 -> 0x0201 */
+#      define COMPILER_NAME2    " 2.01"
 #    elif ((__TURBOC__ >= 0x018d) && (__TURBOC__ <= 0x0200)) /* James: 0x0200 */
-        " 2.0",
+#      define COMPILER_NAME2    " 2.0"
 #    elif (__TURBOC__ > 0x0100)
-        " 1.5",                    /* James:  0x0105? */
+#      define COMPILER_NAME2    " 1.5"          /* James:  0x0105? */
 #    else
-        " 1.0",                    /* James:  0x0100 */
+#      define COMPILER_NAME2    " 1.0"          /* James:  0x0100 */
 #    endif
 #  endif
 #elif defined(MSC)
-      "Microsoft C ",
-#  ifdef _MSC_VER
-#    if (_MSC_VER == 800)
-        "(Visual C++ v1.1)",
-#    elif (_MSC_VER == 850)
-        "(Windows NT v3.5 SDK)",
-#    elif (_MSC_VER == 900)
-        "(Visual C++ v2.0/v2.1)",
-#    elif (_MSC_VER > 900)
-        (sprintf(buf2, "(Visual C++ v%d.%d)", _MSC_VER/100 - 6,
-          _MSC_VER%100/10), buf2),
-#    else
-        (sprintf(buf, "%d.%02d", _MSC_VER/100, _MSC_VER%100), buf),
-#    endif
+#  if defined(_QC) && !defined(_MSC_VER)
+#    define COMPILER_NAME1      "Microsoft Quick C"
+#    define COMPILER_NAME2      ""      /* _QC is defined as 1 */
 #  else
-      "5.1 or earlier",
+#    define COMPILER_NAME1      "Microsoft C "
+#    ifdef _MSC_VER
+#      if (_MSC_VER == 800)
+#        define COMPILER_NAME2  "8.0/8.0c (Visual C++ 1.0/1.5)"
+#      else
+#        define COMPILER_NAME2 \
+           (sprintf(buf, "%d.%02d", _MSC_VER/100, _MSC_VER%100), buf)
+#      endif
+#    else
+#      define COMPILER_NAME2    "5.1 or earlier"
+#    endif
 #  endif
 #else
-      "unknown compiler", "",
+#    define COMPILER_NAME1      "unknown compiler"
+#    define COMPILER_NAME2      ""
 #endif
 
-      "MS-DOS",
+/* Define the OS name and memory environment strings */
+#if defined(__WATCOMC__) || defined(__TURBOC__) || defined(MSC) || \
+    defined(__GNUC__)
+#  define OS_NAME1      "\nMS-DOS"
+#else
+#  define OS_NAME1      "MS-DOS"
+#endif
 
 #if (defined(__GNUC__) || (defined(__WATCOMC__) && defined(__386__)))
-      " (32-bit)",
+#  define OS_NAME2      " (32-bit)"
 #elif defined(M_I86HM) || defined(__HUGE__)
-      " (16-bit, huge)",
+#  define OS_NAME2      " (16-bit, huge)"
 #elif defined(M_I86LM) || defined(__LARGE__)
-      " (16-bit, large)",
+#  define OS_NAME2      " (16-bit, large)"
 #elif defined(M_I86MM) || defined(__MEDIUM__)
-      " (16-bit, medium)",
+#  define OS_NAME2      " (16-bit, medium)"
 #elif defined(M_I86CM) || defined(__COMPACT__)
-      " (16-bit, compact)",
+#  define OS_NAME2      " (16-bit, compact)"
 #elif defined(M_I86SM) || defined(__SMALL__)
-      " (16-bit, small)",
+#  define OS_NAME2      " (16-bit, small)"
 #elif defined(M_I86TM) || defined(__TINY__)
-      " (16-bit, tiny)",
+#  define OS_NAME2      " (16-bit, tiny)"
 #else
-      " (16-bit)",
+#  define OS_NAME2      " (16-bit)"
 #endif
 
+/* Define the compile date string */
 #ifdef __DATE__
-      " on ", __DATE__
+#  define COMPILE_DATE " on " __DATE__
 #else
-      "", ""
+#  define COMPILE_DATE ""
 #endif
-    );
+
+    printf(CompiledWith, COMPILER_NAME1, COMPILER_NAME2,
+           OS_NAME1, OS_NAME2, COMPILE_DATE);
 
 } /* end function version_local() */
 #endif /* !WINDLL */
 
 
-#if __DJGPP__ == 2
-int _is_executable (const char *p,int i,const char *q)
+#if 0 /* inserted here for future use (clearing of archive bits) */
+#if (defined(__GO32__) && (!defined(__DJGPP__) || (__DJGPP__ < 2)))
+
+#include <errno.h>
+int volatile _doserrno;
+
+unsigned _dos_setfileattr(char *name, unsigned attr)
+{
+#if 0   /* stripping of trailing '/' is not needed for zip-internal use */
+    unsigned namlen = strlen(name);
+    char *i_name = alloca(namlen + 1);
+
+    strcpy(i_name, name);
+    if (namlen > 1 && i_name[namlen-1] == '/' && i_name[namlen-2] != ':')
+        i_name[namlen-1] = '\0';
+    asm("movl %0, %%edx": : "g" (i_name));
+#else
+    asm("movl %0, %%edx": : "g" (name));
+#endif
+    asm("movl %0, %%ecx": : "g" (attr));
+    asm("movl $0x4301, %eax");
+    asm("int $0x21": : : "%eax", "%ebx", "%ecx", "%edx", "%esi", "%edi");
+    _doserrno = 0;
+    asm("jnc 1f");
+    asm("movl %%eax, %0": "=m" (_doserrno));
+    switch (_doserrno) {
+    case 2:
+    case 3:
+           errno = ENOENT;
+           break;
+    case 5:
+           errno = EACCES;
+           break;
+    }
+    asm("1:");
+    return (unsigned)_doserrno;
+}
+
+#endif /* DJGPP v1.x */
+#endif /* never (not yet used) */
+
+
+#if (defined(__DJGPP__) && (__DJGPP__ >= 2))
+
+/* Disable determination of "x" bit in st_mode field for [f]stat() calls. */
+int _is_executable (const char *path, int fhandle, const char *ext)
 {
     return 0;
 }
+
+/* Prevent globbing of filenames.  This gives the same functionality as
+ * "stubedit <program> globbing=no" did with DJGPP v1.
+ */
+#ifndef USE_DJGPP_GLOB
+char **__crt0_glob_function(char *_arg)
+{
+    return NULL;
+}
 #endif
 
+/* Reduce the size of the executable and remove the functionality to read
+ * the program's environment from whatever $DJGPP points to.
+ */
+#if !defined(USE_DJGPP_ENV) || defined(UTIL)
+void __crt0_load_environment_file(char *_app_name)
+{
+}
+#endif
+
+#endif /* __DJGPP__ >= 2 */
+
+
 #if defined(_MSC_VER) && _MSC_VER == 700
 
 /*
